/*
	serial.C - UART functions for the MSP430x149

  These functions provide interrupt or non-interrupt RS232 serial
  communication.
  
  Copyright (C) 2001 John Price <linux-guru@gcfl.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */

#include "gendef.h"
#include <msp430x14x.h>
#include "serial.h"

////////////////////////////////////////////////////////////////
//   UART0 variables
////////////////////////////////////////////////////////////////
#ifdef UART0_INT_DRIVEN
#ifdef USE_UART0
char uart0_rx_buf[UART0_RX_BUFSIZE];
char uart0_tx_buf[UART0_TX_BUFSIZE];

uint8 uart0_rx_head, uart0_rx_tail;
uint8 uart0_tx_head, uart0_tx_tail;
uint8 uart0_flags;
#endif // USE_UART0
#endif // UART0_INT_DRIVEN

////////////////////////////////////////////////////////////////
//   UART1 variables
////////////////////////////////////////////////////////////////
#ifdef UART1_INT_DRIVEN
#ifdef USE_UART1
char uart1_rx_buf[UART1_RX_BUFSIZE];
char uart1_tx_buf[UART1_TX_BUFSIZE];

uint8 uart1_rx_head, uart1_rx_tail;
uint8 uart1_tx_head, uart1_tx_tail;
uint8 uart1_flags;
#endif //USE_UART1
#endif // UART1_INT_DRIVEN


////////////////////////////////////////////////////////////////
//   UART0 functions
////////////////////////////////////////////////////////////////
#ifdef USE_UART0
#ifdef UART0_INT_DRIVEN
void UART0_ClearTxBuffer(void)
{
  uart0_tx_head = 0;
  uart0_tx_tail = 0;
}

void UART0_ClearRxBuffer(void)
{
  uart0_rx_head = 0;
  uart0_rx_tail = 0;
}

bool UART0_TxBufferEmpty(void)
{
  return (uart0_tx_head == uart0_tx_tail);
}

bool UART0_RxBufferEmpty(void)
{
  return (uart0_rx_head == uart0_rx_tail);
}

bool UART0_TxBufferFull(void)
{
  uint8 next = uart0_tx_head + 1;
  if (next >= UART0_TX_BUFSIZE)
    next = 0;

  return (next == uart0_tx_tail);
}

bool UART0_RxBufferFull(void)
{
  uint8 next = uart0_rx_head + 1;
  if (next >= UART0_RX_BUFSIZE)
    next = 0;

  return (next == uart0_rx_tail);
}
#endif //UART0_INT_DRIVEN


bool UART0_RXready(void)
{
#ifdef UART0_INT_DRIVEN
  return (uart0_rx_head != uart0_rx_tail);
#else
  return ((IFG1 & URXIFG0) != 0);
#endif
}

char UART0_getchar(void)
{
#ifdef UART0_INT_DRIVEN
  char c;

  /* wait forever for character to arrive */
  while (uart0_rx_tail == uart0_rx_head);

  c = uart0_rx_buf[uart0_rx_head];
  if (++uart0_rx_head >= UART0_RX_BUFSIZE)
    uart0_rx_head = 0;

  return c;
#else
  for (;;)
  {
    if (U0RCTL & RXERR)
    {
      // if you want to answer receive errors, do it here.
      U0RCTL &= ~(FE+PE+OE+BRK+RXERR);
    }
    else
    {
      if (IFG1 & URXIFG0)
      {  // we've received a character
        return U0RXBUF;
      }
    }
  }
#endif //UART0_INT_DRIVEN
}

char UART0_putchar(char c)
{
#ifdef UART0_INT_DRIVEN
  uint8 next;

  next = uart0_tx_head + 1;
  if (next >= UART0_TX_BUFSIZE)
    next = 0;

  /* wait forever until buffer is not full */
  while (next == uart0_tx_tail);

  uart0_tx_buf[uart0_tx_head] = c;  // store character in buffer
  uart0_tx_head = next;

  // if not already transmitting, then start transmit interrupt
  if (!(uart0_flags & UART_TRANSMITTING))
  {
    U0TXBUF = uart0_tx_buf[uart0_tx_tail];
    if (++uart0_tx_tail >= UART0_TX_BUFSIZE)
      uart0_tx_tail = 0;
    uart0_flags |= UART_TRANSMITTING;      /* set transmitting flag */
  }
  return c;
#else
  // wait for other character to transmit
  while (!(IFG1 & UTXIFG0));
  U0TXBUF = c;
  return c;
#endif //UART0_INT_DRIVEN
}

void UART0_putstring(char *s)
{
  while (*s) UART0_putchar(*s++);
}

#endif // USE_UART0


////////////////////////////////////////////////////////////////
//   UART1 functions
////////////////////////////////////////////////////////////////
#ifdef USE_UART1
#ifdef UART1_INT_DRIVEN
void UART1_ClearTxBuffer(void)
{
  uart1_tx_head = 0;
  uart1_tx_tail = 0;
}

void UART1_ClearRxBuffer(void)
{
  uart1_rx_head = 0;
  uart1_rx_tail = 0;
}

bool UART1_TxBufferEmpty(void)
{
  return (uart1_tx_head == uart1_tx_tail);
}

bool UART1_RxBufferEmpty(void)
{
  return (uart1_rx_head == uart1_rx_tail);
}

bool UART1_TxBufferFull(void)
{
  uint8 next = uart1_tx_head + 1;
  if (next >= UART1_TX_BUFSIZE)
    next = 0;

  return (next == uart1_tx_tail);
}

bool UART1_RxBufferFull(void)
{
  uint8 next = uart1_rx_head + 1;
  if (next >= UART1_RX_BUFSIZE)
    next = 0;

  return (next == uart1_rx_tail);
}
#endif //UART1_INT_DRIVEN


bool UART1_RXready(void)
{
#ifdef UART1_INT_DRIVEN
  return (uart1_rx_head != uart1_rx_tail);
#else
  return ((IFG2 & URXIFG1) != 0);
#endif
}

char UART1_getchar(void)
{
#ifdef UART1_INT_DRIVEN
  char c;

  /* wait forever for character to arrive */
  while (uart1_rx_tail == uart1_rx_head);

  c = uart1_rx_buf[uart1_rx_head];
  if (++uart1_rx_head >= UART1_RX_BUFSIZE)
    uart1_rx_head = 0;

  return c;
#else
  for (;;)
  {
    if (U1RCTL & RXERR)
    {
      // if you want to answer receive errors, do it here.
      U1RCTL &= ~(FE+PE+OE+BRK+RXERR);
    }
    else
    {
      if (IFG2 & URXIFG1)
      {  // we've received a character
        return U1RXBUF;
      }
    }
  }
#endif //UART1_INT_DRIVEN
}


char UART1_putchar(char c)
{
#ifdef UART1_INT_DRIVEN
  uint8 next;

  next = uart1_tx_head + 1;
  if (next >= UART1_TX_BUFSIZE)
    next = 0;

  /* wait forever until buffer is not full */
  while (next == uart1_tx_tail);

  uart1_tx_buf[uart1_tx_head] = c;  // store character in buffer
  uart1_tx_head = next;

  // if not already transmitting, then start transmit interrupt
  if (!(uart1_flags & UART_TRANSMITTING))
  {
    U1TXBUF = uart1_tx_buf[uart1_tx_tail];
    if (++uart1_tx_tail >= UART1_TX_BUFSIZE)
      uart1_tx_tail = 0;
    uart1_flags |= UART_TRANSMITTING;      /* set transmitting flag */
  }
  return c;
#else
  // wait for other character to transmit
  while (!(IFG2 & UTXIFG1));
  U1TXBUF = c;
  return c;
#endif //UART1_INT_DRIVEN
}

void UART1_putstring(char *s)
{
  while (*s) UART1_putchar(*s++);
}
#endif // USE_UART1



#ifdef UART0_INT_DRIVEN
#ifdef USE_UART0
/* RS232 receive interrupt service routine */
interrupt [UART0RX_VECTOR] void UART0_RX_interrupt(void)
{
  uint8 next;

  /* if we received a bad character, just clear flags and discard. */
  if (U0RCTL & RXERR)
    U0RCTL &= ~(FE+PE+OE+BRK+RXERR);
//  else
  {
    next = uart0_rx_tail + 1;
    if (next >= UART0_RX_BUFSIZE)
      next = 0;

    if (next != uart0_rx_head)
    {
      uart0_rx_buf[uart0_rx_tail] = U0RXBUF;
      uart0_rx_tail = next;
      uart0_flags &= ~UART_RX_OVERFLOW;     /* clear RX buffer full flag */
    }
    else
    {
      uart0_flags |= UART_RX_OVERFLOW;      /* set RX buffer full flag */
    }
  }
}


/* RS232 transmit interrupt service routine */
interrupt [UART0TX_VECTOR] void UART0_TX_interrupt(void)
{
  if (uart0_tx_tail != uart0_tx_head) /* check if buffer is empty */
  {
    U0TXBUF = uart0_tx_buf[uart0_tx_tail];
    if (++uart0_tx_tail >= UART0_TX_BUFSIZE)
      uart0_tx_tail = 0;
    uart0_flags |= UART_TRANSMITTING;       /* set transmitting flag */
  }
  else
  {
    uart0_flags &= ~UART_TRANSMITTING;      /* clear transmitting flag */
  }
}
#endif //USE_UART0
#endif // UART0_INT_DRIVEN


#ifdef UART1_INT_DRIVEN
#ifdef USE_UART1
/* RS232 receive interrupt service routine */
interrupt [UART1RX_VECTOR] void UART1_RX_interrupt(void)
{
  uint8 next;

  /* if we received a bad character, just clear flags and discard. */
  if (U1RCTL & RXERR)
    U1RCTL &= ~(FE+PE+OE+BRK+RXERR);
//  else
  {
    next = uart1_rx_tail + 1;
    if (next >= UART1_RX_BUFSIZE)
      next = 0;

    if (next != uart1_rx_head)
    {
      uart1_rx_buf[uart1_rx_tail] = U1RXBUF;
      uart1_rx_tail = next;
      uart1_flags &= ~UART_RX_OVERFLOW;     /* clear RX buffer full flag */
    }
    else
    {
      uart1_flags |= UART_RX_OVERFLOW;      /* set RX buffer full flag */
    }
  }
}


/* RS232 transmit interrupt service routine */
interrupt [UART1TX_VECTOR] void UART1_TX_interrupt(void)
{
  if (uart1_tx_tail != uart1_tx_head) /* check if buffer is empty */
  {
    U1TXBUF = uart1_tx_buf[uart1_tx_tail];
    if (++uart1_tx_tail >= UART1_TX_BUFSIZE)
      uart1_tx_tail = 0;
    uart1_flags |= UART_TRANSMITTING;       /* set transmitting flag */
  }
  else
  {
    uart1_flags &= ~UART_TRANSMITTING;      /* clear transmitting flag */
  }
}
#endif //USE_UART1

#endif //UART1_INT_DRIVEN

